#!/bin/bash
# 
# NOTE TO COMMUNITY SCRIPT REPOSITORY CONTRIBUTORS:
#   Got any interesting software group ideas? Add it to this script.
#
# Title:
#   Quick Software Installer for Ubuntu BETA
#
# Description:
#    This simple bash script installs and sets up certain repositories.
#    The software installed by the script is useful for the author(s).
#
# Author: 
#   Jonathan Ben-Joseph <jbenjos@fau.edu>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

CODENAME="`lsb_release -a | awk -F":" '/Codename/ { print substr($2,2) }'`"

if [[ $EUID -ne 0 ]]; then
   echo "Error: This script must be run as a superuser. Did you prefix with sudo or are you root?"
   echo 
   exit 1
fi

echo "Quick Software Installer for Ubuntu (BETA)"
echo
echo "1. Install font collection"
echo "2. Install media codecs, Flash, Java and DVD support (requires accepting EULA)"
echo "3. Install Ubuntu development toolkit"
echo "4. Install additional themes and wallpapers for Gnome"
echo "Q. Quit without doing anything"
echo
echo "Please select software to install by entering the menu option number."
echo "You may select more then one menu option. (eg: 12)"
read -p "Enter option: " IN 

echo

if [[ "$IN" =~ ^[qQ] ]];  then
     echo "Goodbye!"
     exit 0
fi

if [[ "$IN" == *3* ]]; then
     echo "Development toolkit requires pre-installation information!"
     read -p "Please provide your full name: " NAME

     read -p "Please provide your e-mail address: " EMAIL
     echo
fi

echo "STARTING INSTALLATION"
echo "Please be patient, it takes awhile!"

if [[ "$IN" == *1* ]]; then
     echo -n "Installing font collection (100+ fonts)..."
     : apt-get -y install ttf-liberation ttf-sjfonts ttf-gentium ttf-larabie-deco ttf-larabie-straight ttf-larabie-uncommon ttf-georgewilliams ttf-dustin
     echo "  Done!"   
fi

if [[ "$IN" == *3* ]]; then
     echo -n "Installing development toolkit..."
     : apt-get -y install devscripts build-essential fakeroot debhelper gnupg pbuilder dh-make bzr
     
     echo >> ~/.bashrc
     echo "# Debian packager information (automatically added by quick-installer.sh)" >> ~/.bashrc
     echo "export DEBFULLNAME=\""$NAME"\"" >> ~/.bashrc
     echo "export DEBEMAIL=\""$EMAIL"\"" >> ~/.bashrc
     
     echo "COMPONENTS=\"main restricted universe multiverse\"" > ~/.pbuilderrc
     pbuilder create &> /dev/null
     
     echo "  Done!"   
fi

if [[ "$IN" == *4* ]]; then
     echo -n "Installing additonal themes and wallpapers for Gnome..."
     : apt-get -y install community-themes edgy-community-wallpapers ubuntustudio-wallpapers ubuntustudio-look
     echo "  Done!"   
fi

if [[ "$IN" == *2* ]]; then
     echo -n "Installing media codecs, Flash, Java and DVD support..."
     : wget http://www.medibuntu.org/sources.list.d/${CODENAME}.list --output-document=/etc/apt/sources.list.d/medibuntu.list
     if [ "$?" -ne "0" ] ; then
          echo " Failed!"
          echo "Error: Could not download Medibuntu repository list. Your distro ${CODENAME} might not be supported."
     else 
          : apt-get update
          : apt-get -y --force-yes install medibuntu-keyring
          : apt-get update
          # Interactivity required because of EULAs
          apt-get -y install ubuntu-restricted-extras libdvdcss2
          echo "  Done!"
     fi       
fi

echo "All installations finished."
