#!/bin/bash
# 
# Title:
#   Ubuntu Packaging Toolkit Installer
#
# Description:
#    This simple bash script prepares a new machine (using the bash shell) for Ubuntu packaging.
# 
# Author: 
#   Jonathan Ben-Joseph <jbenjos@fau.edu>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

echo "Ubuntu Packaging Installer"
echo "This script prepares a new machine (using the bash shell) for Ubuntu packaging."
echo

if [[ $EUID -ne 0 ]]; then
   echo "Error: This script must be run as a superuser. Did you prefix with sudo or are you root?"
   echo 
   exit 1
fi

read -p "Please provide your full name: " NAME

read -p "Please provide your e-mail address: " EMAIL
echo

echo "STARTING INSTALLATION PROCESS"
echo -n "Installing packages..."
: apt-get -y install devscripts build-essential fakeroot debhelper gnupg pbuilder dh-make bzr
echo "  Done!"

echo -n "Setting up Debian packager information enviromental variables..."
echo >> ~/.bashrc
echo "# Debian packager information (automatically added by ubuntu-devel.sh)" >> ~/.bashrc
echo "export DEBFULLNAME=\""$NAME"\"" >> ~/.bashrc
echo "export DEBEMAIL=\""$EMAIL"\"" >> ~/.bashrc
echo " Done!"

echo -n "Setting up pbuilder environment (this takes awhile)..."
echo "COMPONENTS=\"main restricted universe multiverse\"" > ~/.pbuilderrc
: pbuilder create
echo " Done!"

echo
echo "Everything should be set up!"

