#!/bin/bash
# 
# Title:
#   Useful Aliases for Ubuntu / Debian
#
# Description:
#    This simple bash script sets up a a few useful bash aliases for Debian based operating systems.
# 
# Author: 
#   Jonathan Ben-Joseph <jbenjos@fau.edu>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# s
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

echo >> ~/.bashrc
echo "# Useful aliases for Ubuntu (added by useful-aliases.sh)" >> ~/.bashrc

echo "PACKAGING RELATED ALIASES"

echo "Adding 'upgrade' alias (updates and upgrades Ubuntu)..." 
echo "alias upgrade=\"sudo apt-get update; sudo apt-get -y dist-upgrade; sudo apt-get -y autoclean\"" >> ~/.bashrc

echo "Adding 'upgrade-unsafe' alias (updates and upgrades Ubuntu and does not check for signed packages)..." 
echo "alias upgrade-unsafe=\"sudo apt-get update; sudo apt-get --force-yes -y dist-upgrade; sudo apt-get -y autoclean\"" >> ~/.bashrc

echo "Adding 'show' alias (shows information on a package specified as an arguement)..." 
echo "alias show=\"apt-cache show\"" >> ~/.bashrc

echo "Adding 'pkgsch' alias (searches for packages matching the given regxp)..." 
echo "alias pkgsch=\"apt-cache search\"" >> ~/.bashrc

echo "Adding 'ins' alias (installs the package(s) specified and any dependencies)..." 
echo "alias ins=\"sudo apt-get -y install\"" >> ~/.bashrc

echo "Adding 'rem' alias (removes the package(s) specified and any dependencies)..." 
echo "alias rem=\"sudo apt-get -y remove\"" >> ~/.bashrc

echo
echo "NAVIGATION RELATED ALIASES"
echo "Adding '..' alias (shortcut for \"cd ..\")..."
echo "alias ..=\"cd ..\"" >> ~/.bashrc

echo "Adding '...' alias (shortcut for \"cd ../..\")..."
echo "alias ...=\"cd ../..\"" >> ~/.bashrc

echo "Adding '....' alias (shortcut for \"cd ../../..\")..."
echo "alias ....=\"cd ../../..\"" >> ~/.bashrc

echo "Adding '.....' alias (shortcut for \"cd ../../../..\")..."
echo "alias .....=\"cd ../../../..\"" >> ~/.bashrc

echo
echo "OTHER ALIASES"
echo "Adding 'svim' alias (shortcut for \"sudo vim\")..."
echo "alias svim=\"sudo vim\"" >> ~/.bashrc

echo "Adding 'fcow' alias (shortcut for \"fortune | cowsay\")..."
echo "alias fcow=\"fortune | cowsay\"" >> ~/.bashrc

echo "Adding 'serv' function (shortcut for \"sudo /etc/init.d/\")..."
echo "function serv { sudo /etc/init.d/\$1 \$2; }" >> ~/.bashrc

echo "All the aliases have been added to ~/.bashrc"
echo "You can go and add your own too!"
echo "Reload your configuration [ . ~/.bashrc ] or restart your session to see the changes."; echo
