#!/bin/bash
# 
# Title:
#   Gstreamer MP3 Player Script
#
# Description:
#   "Frontend" to gstreamer that lets you easily play mp3s from your bash shell
# 
# Author: 
#   David Futcher <bobbo@ubuntu.com>
#
# Howto:
#   Import the file (. mp3-player.sh) and run:
#       gst-play <filename> to play a file
#       gst-stop to stop playing
#       gst-status to get current player status
#
# Warning: May explode in massive flames if you are using another application
# that uses gstreamer. Do not use if you are doing anything particularly 
# important that uses video/sound!
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

export GST_BIN="gst-launch-0.10"
export GST_ARGS="--gst-debug-level=0"
export GST_PIPELINE="! decodebin ! audioconvert ! alsasink"
export NOW_PLAYING="NULL"

function gst-stop
{
    kill $(pidof $GST_BIN) > /dev/null 2> /dev/null 0> /dev/null
    export NOW_PLAYING="NULL"
}

function gst-play
{
    gst-stop
    FILE=$1
    ${FILE//\\/} #Strip out backspaces or gstreamer complains
    $GST_BIN $GST_ARGS filesrc location="$FILE" $GST_PIPELINE > /dev/null 2> /dev/null 0> /dev/null &
    export NOW_PLAYING=$FILE
}

function gst-status
{
    if [ "$NOW_PLAYING" == "NULL" ]
    then
        echo "Idle"
    else
        echo "Playing: "$NOW_PLAYING
    fi
}
