#!/usr/bin/env python
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.


import sys, os
import string

#
#    If you're like me, you change DE/WM's a lot, and you don't use a Login Manager (GDM, KDM, etc)
#    You'll find that you have to edit your $HOME/.xinitrc a lot.
#    I made this small script to help me change WM's on the fly without going into VIM/Nano etc.
#    So I'd though I'd share it with the world. :) (P.S. It might be more useful if you drop it into your $PATH)
#
#    How To Use: XChange [SUPPORTED_WINDOW_MANAGER_HERE] [OPTIONAL -cc (Change Commands)]
#
#    Copyright 2008 (c) Huandari Lopez <hlopez137@gmail.com>
#    Version 1.0
#


xinitrc = os.path.join(os.environ["HOME"], ".xinitrc")

# Put your default optional commands here. xset -b mutes that annoying pc speaker beep.
optionalCmds = ["xset -b"]

# You can add/remove to the following lists but
# Keep these two lists in sync!
windowManagers = ["openbox-session", "startkde", "xfce4-session", "gnome-session"]
windowManagerNames = ["Openbox", "KDE", "Xfce", "GNOME"]

def changeX(mode, cmds=None):
    file = None
    error = None
    try:
        file = open(xinitrc, 'w')
        file.write("#!/bin/bash/\n") # The shebang line

        if cmds == None:
            cmds = optionalCmds[:]

        for i in range(len(cmds)):

            file.write("%s\n" % cmds[i])

        file.write("exec %s\n" % windowManagers[mode])

    except (ValueError, IOError), e:
        print "An error has occured: %s." % e
        error = True
    finally:
        if error == None:
            print ":: Changed .xinitrc to execute %s." % windowManagerNames[mode]
        if file != None:
            file.close()

def displayCmds():
    number = 0
    if len(optionalCmds) == 0:
        print "There's no more commands left!"
        return
    for i in range(len(optionalCmds)):
        print "%i. %s" % (number+1, optionalCmds[i])
        number += 1

def changeCmds():
    displayCmds()

    choice = str(raw_input("Do you want to add or remove [A|R]? "))
    if string.lower(choice) == "r":

        cont = True

        while cont:
            if len(optionalCmds) == 0:  # If there are no more commands in the optionalCmds list then exit the loop.
                break;
            remChoice = raw_input("Enter the number that you want to delete: ")

            if int(remChoice)-1 in range(len(optionalCmds)):
                # Ok, the input is valid, delete the item from the list!
                del optionalCmds[int(remChoice)-1]

            contConfirm = raw_input("Do you want to continue removing? ")

            if string.lower(contConfirm) in ("yes","y","1"):
                displayCmds()
                continue
            else:
                print ":: Continuing main loop..."
                cont = False

    elif string.lower(choice) == "a":
        cont = True
        while cont:
            newCommand = raw_input("Enter the command you want to add: ")
            optionalCmds.append(str(newCommand))
            displayCmds()
            contConfirm = raw_input("Do you want to continue adding? ")
            if string.lower(contConfirm) in ("yes", "y", "1"):
                continue
            else:
                print ":: Continuing main loop..."
                cont = False

def main(mode=0, cmds=None):    # The cmds arguement is used for my Qt GUI frontend.
    if mode >= 0 and mode <= 3:
        try:
            if string.lower(sys.argv[2]) == "-cc":
                changeCmds()
        except (IndexError, ValueError):
            pass

        if cmds == None:
            cmds = optionalCmds[:]

        changeX(mode, cmds)
    else:
        print "Wrong input, not changing anything."



if __name__ == "__main__":

    try:
        if string.lower(sys.argv[1]) == "openbox" or string.lower(sys.argv[1]) == "0":
            print ":: Preparing to change .xinitrc to execute Openbox..."
            main(0)
        elif string.lower(sys.argv[1]) == "kde" or string.lower(sys.argv[1]) == "1":
            print ":: Preparing to change .xinitrc to execute KDE..."
            main(1)
        elif string.lower(sys.argv[1]) == "xfce" or string.lower(sys.argv[1]) == "2":
            print ":: Preparing to change .xinitrc to execute Xfce..."
            main(2)
        elif string.lower(sys.argv[1]) == "gnome" or string.lower(sys.argv[1]) == "3":
            print ":: Preparing to change .xinitrc to execute GNOME..."
            main(3)
        else:
            print "Unrecognized command, try again."
    except (IndexError, ValueError):
        print "No command entered!"
