#!/bin/bash
# Pull an existing DBUS session into a remote login

# To use this, you must be logged in to a window manager
# that is compatible with GNOME/KDE on the target machine.
# Type "source ./get_dbus_session.sh" so that the active 
# environment is modified.

# CAVEAT: This script was written to detect the first session
# it finds, if there are multiple logins on the target machine
# it may break the script or connect to the wrong session.

# Any improvements or security fixes are welcomed :)

# Created by Sean Hodges <seanhodges@bluebottle.com>


# Scan these processes for the DBUS session variable
compatiblePrograms=( nautilus kdeinit pulseaudio trackerd )

# Attempt to get a program pid
for index in ${compatiblePrograms[@]}; do
	PID=$(pidof -s ${index})
	if [[ "${PID}" != "" ]]; then
		break
	fi
done
if [[ "${PID}" == "" ]]; then
	echo "Could not detect active login session"
	return 1
fi

QUERY_ENVIRON="$(tr '\0' '\n' < /proc/${PID}/environ | grep "DBUS_SESSION_BUS_ADDRESS" | cut -d "=" -f 2-)"
if [[ "${QUERY_ENVIRON}" != "" ]]; then
	export DBUS_SESSION_BUS_ADDRESS="${QUERY_ENVIRON}"
	echo "Connected to session:"
	echo "DBUS_SESSION_BUS_ADDRESS=${DBUS_SESSION_BUS_ADDRESS}"
else
	echo "Could not find dbus session ID in user environment."
	return 1
fi

return 0
