#!/usr/bin/env python
# Poll for an internet connection before starting programs
# Add this to your session startup (In Gnome: System -> Preferences -> Sessions)
# Set the programs this script should start in "programList" below, and remove
# them from your session startup so that they do not start regardless.

# Created by Sean Hodges <seanhodges@bluebottle.com>

import os
import time
import gtk
import threading

# Programs to launch when net connection is available
programList = [
	"pidgin",
	"xchat-gnome",
	"evolution"
]

class NetPoll(threading.Thread):
	checkInterval = 5 # Delay before each check
	checkTarget = "www.google.com" # Public URL to ping when checking on-line status
	checkTimeout = 1 # Seconds before each ping is abandoned (increase if network suffers from high latency)
	autoLaunch = True # Automatically launch programs without notification

	parent = None

	def __init__(self, parent):
		threading.Thread.__init__(self)
		self.parent = parent

	def run(self):
		# Ping Google until connection is established
		response = 1
		while response != 0:
			response = os.system("ping -q -c 1 -w %i %s > /dev/null" % (self.checkTimeout, self.checkTarget))
			if response != 0:
				time.sleep(self.checkInterval)

		if self.autoLaunch:
			self.parent.launchPrograms()
		else:
			self.parent.set_tooltip("Programs are ready to start...")
			self.parent.dialogMsg = "The following programs are ready to start:"
			self.parent.set_from_stock(gtk.STOCK_CONNECT)
	

class NetSession(gtk.StatusIcon):
	dialogMsg = "The following programs are waiting for an Internet connection:"
	poll = None

	def __init__(self):
		gtk.StatusIcon.__init__(self)
		self.manager = gtk.UIManager()
		self.set_from_stock(gtk.STOCK_DISCONNECT)
		self.set_tooltip("Programs are waiting for an Internet connection before starting...")
		self.set_visible(True)
		self.connect("activate", self.on_activate)
		self.connect('popup-menu', self.on_popup_menu)
		self.poll = NetPoll(self)
		# Start the network polling
		self.poll.start()
	
	def launchPrograms(self):
		for program in programList:
			os.system("which %s > /dev/null && %s &" % (program, program))
		self.close()
	
	def on_activate(self, widget, data=None):
		dialogText = "%s\n\n" % self.dialogMsg
		for program in programList:
			dialogText += "* %s\n" % program
		dialogText += "\nWould you like to start them now?"
		launch = os.system("zenity --question --text='%s'" % dialogText)
		if launch == 0:
			self.launchPrograms()

	def on_popup_menu(self, button, widget, data=None):
		self.on_activate(widget, data)

	def close(self):
		gtk.main_quit()
		exit(0)


if __name__ == "__main__":
	NetSession()
	gtk.gdk.threads_init() 
	gtk.main()
