#!/bin/bash
#
# This script continually checks for a network connection and plays a
# sound when a connection is detected.
#
# The network-test.sh script must be available on the $PATH.

#set -x

#defalts; change at will
pingHost=yahoo.com
soundFile="/home/scott/misc/sounds/MacOS Classic/Quack.wav"
soundProg=play #part of the sox package


###########################################################
thisName="$(basename "$0")"
count=0
tries="ping"
titleEscS='\033]0;' # titlebar escape code - start
titleEscE='\007'    # titlebar escape code - end
color1="\033[4;34m"
color2="\033[0;31m"
color3="\033[0;33m"
transparent="\033[0m" #Transparent - don't change
termWidth=
count2=

function usage {
    echo "$thisName"
    echo "Usage: $thisName [host-to-ping [sound-to-play [program-to-play-sound]]]"
    exit 1
}

function xtermDetect {
    if [[ ! $TERM =~ "xterm" ]]; then
        titleEscS=
        titleEscE=
        color1=
        color2=
        color3=
        transparent=
        return 1
    else return 0
    fi
}

function doTest {
    local status
    count2=13 # Length of Monitoring string
    
    echo -e "${color3}$(date '+%r%t')${transparent} Monitoring ${color1}$pingHost${transparent}..."
    echo -ne "Monitoring: ${color3}»${transparent}"

    while true; do
        ((count++))
        if [ $count2 -ge `echo "$termWidth - 2" | bc` ]; then
            echo -en "${color3}«${transparent}\n            ${color3}»${transparent}"
            count2=13
        fi
        echo -n '.'
        ((count2++))
        [ $count -gt 1 ] && tries="pings"
        ping -qc 1 "$pingHost" >/dev/null 2>&1
        status=$?
        [ $status -ne 0 ] && netDown
        sleep 30
    done
}

function netDown {
    ## Verify that the network is, in fact, down
    ping -qc 1 "$pingHost" >/dev/null 2>&1
    local status=$?
    [ $status -eq 0 ] && return 0
    
    # Looks like it's really down
    echo -e "${color3}«${transparent}\n${color2}$(date '+%r%t')${transparent} ${color3}After $count $tries, ${color1}${pingHost}${color3} is unreachable.${transparent}"
    for i in `seq 5`; do
        "$soundProg" "$soundFile" >/dev/null 2>&1
    done &
    network-test.sh "$pingHost"
    status=$?
    [ $status -ne 0 ] && exit $status
    setTitlebar
    count=0
    count2=13
    tries=ping
    echo -en "Monitoring: ${color3}»${transparent}"
}

function cancel {
    echo -e "${color3}«${transparent}\nCurrent ping count: $count. Exiting."
    onExit
    exit 0
}

function onExit {
    ## Re-enable echo
    stty echo
    ## Show the cursor
    echo -ne '\E[?25h'
}

function setTitlebar {
    xtermDetect && echo -ne "${titleEscS}Monitoring $pingHost... Currently available${titleEscE}"
}

function windowResized {
    termWidth="$(stty --all | grep columns | sed 's:^.*columns \([0-9]*\);.*$:\1:')"
}

function main {
    [[ $1 == "--help" ]] && usage
    [ -n "$1" ] && pingHost="$1"; shift
    [ -n "$1" ] && soundFile="$1"; shift
    [ -n "$1" ] && soundprog="$1"
    
    # Set the titlebar text
    setTitlebar
    
    trap cancel INT TERM HUP
    trap windowResized WINCH
    
    ## Hide the cursor
    echo -ne '\E[?25l'
    ## disable echoing to avoid messing up the input
    stty -echo

    # Find the terminal width
    windowResized
    
    clear
    doTest
}

main "$@"
