#!/bin/bash
# Title:
#    Network Test
# Description:
# This script continually checks for a network connection and plays a
# sound when a connection is detected.

#defalts; change at will
pingHost=yahoo.com
soundFile="$HOME/misc/sounds/MacOS Classic/Wild Eep.wav"
soundProg=play # The play command is part of the sox package.


###########################################################
thisName="$(basename "$0")"
counter=0
tries="ping"
titleEscS='\033]0;' # titlebar escape code - start
titleEscE='\007'    # titlebar escape code - end
color1="\033[4;34m"
color2="\033[1;32m"
color3="\033[36m"
transparent="\033[0m" #Transparent - don't change
termWidth=

function usage {
    echo "$thisName"
    echo "Usage: $thisName [host-to-ping [sound-to-play [program-to-play-sound]]]"
    exit 1
}

function xtermDetect {
    if [[ ! $TERM =~ "xterm" ]]; then
        titleEscS=
        titleEscE=
        color1=
        color2=
        color3=
        transparent=
        return 1
    else return 0
    fi
}

function doTest {
    local status
    local count2=13 #length of Testing string
    
    echo -e "${color3}$(date '+%r%t')${transparent} Pinging ${color1}$pingHost${transparent}..."
    echo -en "Testing:    ${color3}»${transparent}"

    while true; do
        ((count++))
        if [ $count2 -ge `echo "$termWidth - 2" | bc` ]; then
            echo -en "${color3}«${transparent}\n            ${color3}»${transparent}"
            count2=13
        fi
        echo -n '.'
        ((count2++))
        [ $count -gt 1 ] && tries="pings"
        ping -qc 1 "$pingHost" >/dev/null 2>&1
        status=$?
        if [ $status -eq 0 ]; then success; fi
        sleep 10
    done
}

function success {
    echo -e "${color3}«${trnasparent}\n${color3}$(date '+%r%t')${transparent} ${color2}You now have network access!${transparent} It took $count $tries."
    #for i in 1 2 3; do
    "$soundProg" "$soundFile" >/dev/null 2>&1
    #done
    exit 0
}

function fail {
    echo -e "${color3}«${transparent}\nNo network access after $count $tries."
    exit 1
}

function windowResize {
    termWidth="$(stty --all | grep columns | sed 's:^.*columns \([0-9]*\);.*$:\1:')"
}

function main {
    [[ $1 == "--help" ]] && usage
    [ -n "$1" ] && pingHost="$1"; shift
    [ -n "$1" ] && soundFile="$1"; shift
    [ -n "$1" ] && soundprog="$1"
    
    # Set the titlebar text
    xtermDetect && echo -ne "${titleEscS}Network test - $pingHost${titleEscE}"

    # Find the terminal width
    windowResize
    
    trap fail INT TERM HUP
    trap windowResize WINCH

    doTest
}

main "$@"
