#!/usr/bin/env ruby
# Title:
#   List process by regular expression
# Description:
# This script is a shortcut to listing all processes matching one or more regular expressions.
# When you do ps -ef | grep something, sometimes the grep command will show up, which is 
# undesireable at best and can break things that check the exit status at worst. Additionally,
# It can handle multiple search terms. Run p --help for instructions.

require 'getoptlong'
#require 'rubygems'
#require 'ruby-debug'
#debugger

class Main
  def initialize
    @inverse = false
    @case_insensitive = false
    @user = `whoami`.chomp
    cmd_lnk = if ARGV.length >= 1
                ' '
              else
                ''
              end
    @self_cmd = "ruby #{$0}#{cmd_lnk}#{ARGV.join(' ')}"
    set_options
    ps = `ps -ef`.chomp.split("\n")
    ps = ps.delete_if do |i|
      i =~ Regexp.new("#{@user}.*(ps -ef|#{Regexp.escape(@self_cmd)})$")
    end
    ps = weed_lines(ps)
    exit 1 if ps.length == 0
    puts ps.join("\n")
  end
  
  def weed_lines(l)
    options = if @case_insensitive
                Regexp::IGNORECASE
              else
                nil
              end
    if @inverse
      result = l.delete_if { |i| i =~ Regexp.new(ARGV.join('|'),options) }
    else
      result = l
      ARGV.each do |term|
        result = result.grep(Regexp.new(term,options))
      end
    end
    result
  end
  
  def set_options
    valid_options = [['-h', '--help',    GetoptLong::NO_ARGUMENT],
                     ['-i', '--ignore-case', GetoptLong::NO_ARGUMENT],
                     ['-v', '--invert-match', GetoptLong::NO_ARGUMENT]
                    ]
    options = GetoptLong.new(*valid_options)
    options.each do |opt,arg|
      case opt
        when '-h'
          usage
          exit
        when '-i'
          @case_insensitive = true
        when '-v'
          @inverse = true
      end
    end
  end
  
  def usage
    puts <<END
Shortcut to "ps -ef | grep foo"

  Usage: p [-i] [-v] pattern1 [pattern2 [pattern3 [...]]]
  
    -i, --ingore-case:      Case-insensitive grep
    -v, --invert-match:     Inverse grep: return lines that do NOT match
                            the pattern
  
  Note: The patterns are regular expressions (as interpreted by Ruby).
        Be sure to properly escape any special characters to protect
        them from the shell.
END
  end
end

Main.new if $0 == __FILE__
