#!/bin/bash

# Title:
#   sysres: Linux system restore program.
# Description:
#   Captures a snapshot of currently installed packages and provides an option of restoration.
# Author:
#   LaRoza

#Copyright (C) 2008  LaRoza
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

nochange()
{
    zenity --info --text "Package Backup and Restore Halted -- No Changes Made";
    exit 0;
}

restore()
{
    if [ $UID -eq 0 ]
        then
        name=$(zenity --file-selection --text "Open file:" --filename ~/.system_restore/);

        zenity --question --text "Install all packages?";
        
        if [ $? -eq 0 ] 
        then

            if [ -f $name ]
            then
                zenity --info --text "File exists and ready";
                gksudo dpkg --set-selections < $name; dselect;
            else
                zenity --info --text "Not a file";
                nochange;
            fi

        else
            nochange;
        fi
    else
        zenity --info --text "You are not root";
        exit 0
    fi

    return;  
}
backup()
{
    saveas=$(zenity --entry --text "Save as:");
     
    if [ -e ~/.sysres/$saveas ]
    then  
        if [ -d ~/.sysres/$saveas ]
        then
            zenity --info --text "That is a directory";
            nochange;
        fi

        zenity --question --text "File exists, overwrite?";

        if [ $? -eq 0 ] 
        then
            dpkg --get-selections > ~/.sysres/$saveas;
        else
            exit;
        fi
        
    else
        dpkg --get-selections > ~/.sysres/$saveas;
    fi

    return;
}


main()
{
    ans=$(zenity  --list  --text "Package Backup and Restore" --radiolist  --column "Pick" --column "Option" TRUE "Backup" FALSE "Restore");

    if [ $? -eq 1 ]
    then
        nochange;
    else
        if [ $ans == "Backup" ] 
        then
            backup;
        elif [ $ans == "Restore" ]
        then
            restore;
        else
            nochange;
        fi
    fi
        
}

main;
