#!/bin/bash
# Title:
#    Search tool for command-line using Tracker
# Author:
#    Sean Hodges 
# Date Created:
#    22 Dec, 2007

# Set your default editor using the command: sudo update-alternatives --config editor
# When using vim, you must add "set isfname+=32" to your .vimrc to accept spaces in filenames

mode="whiptail" # Can be "whiptail", "dialog" or "vim"
maxresults=32 # Reduce the search results to this number (whiptail/dialog only)

query=$*

if [ "$query" == "" ]; then
	# No search terms were provided
	echo "Please add some search terms"
	exit 1
fi

counter=0
cmd=""

if [ $mode == "vim" ]; then
	# VIM mode
	results="`tracker-search $query`"
	echo -e "$results" | vim -R -
else
	# Whiptail/Dialog mode
	while read line; do
		let counter=counter+1
		cmd="$cmd $counter ${line// /_}"
		resultlist[$counter]="$line"
		if [ "$counter" -ge "$maxresults" ]; then break; fi
	done < <(tracker-search $query)
	if [ "$counter" -gt 0 ]; then
		$mode --menu "Search Results" 40 120 30 $cmd 2>/tmp/ans
		if [ $? != 0 ]; then 
			rm /tmp/ans
			exit 0
		fi
		selection="`cat /tmp/ans`"
		rm -f /tmp/ans
		editor ${resultlist[$selection]}
	else
		echo "No results found"
	fi
fi
