#!/bin/bash
# Title:
#   Recursive shred
# Description:
#    Makes shred(1) work recursively. Run shred-recursive --help for usage instructions.

EXIT_VAL=0
progName="$(basename "$0")"
verStr="Recursive Shredder 1.1"

FILE_COLOR="\033[4;34m"
ERROR_COLOR="\033[0;31m"
DONE_COLOR="\033[0;32m"
NO_COLOR="\033[0m" #Transparent - don't change

licTxt() {
	echo "Copyright (c) 2006 by Scott Severance"
	echo "Licensed under the terms of the GNU General Public License"
	echo
	echo "This software is provided \"AS-IS\" and carries NO WARRANTY. Use it at your own risk."
}

shredThis() {
	#echo "press any key"
	#read -sn 1
	if [ -d "$1" -a "$1" != "." -a "$1" != ".." ]; then
		if [ -w "$1" -a -x "$1" ]; then
			local oldDir="$(pwd)"
			builtin cd "$1"
			echo -e "Entering directory: ${FILE_COLOR}$(pwd)${NO_COLOR}"
			#pwd
			for i in * .*; do
				if [ "$i" != "." -a "$i" != ".." -a "$i" != "*" ]; then
					shredThis "$i"
				fi
				done
			builtin cd "$oldDir"
			echo -e "Removing directory: ${FILE_COLOR}${1}${NO_COLOR}"
			echo -ne $ERROR_COLOR
			rmdir "$1"
			echo -ne $NO_COLOR
		else
			echo -e "${ERROR_COLOR}Directory ${FILE_COLOR}$1${ERROR_COLOR} is not writable and/or searchable. Skipping.${NO_COLOR}"
			EXIT_VAL=4
		fi
		#echo -n "Returning to "
		#pwd

	elif [ -L "$1" -o -p "$1" -o -S "$1" ]; then # test for symlinks, fifos, and sockets
		if [ -w "$1" ]; then
			echo -ne "Deleting special file ${FILE_COLOR}${1}${NO_COLOR}... "
			echo -ne $ERROR_COLOR
			rm "$1"
			local rslt="$?"
			echo -ne $NO_COLOR
			if [ "$rslt" -ne "0" ]; then
				echo -e "${ERROR_COLOR}### ERROR deleting ${FILE_COLOR}${1}${ERROR_COLOR}: rm exited with status ${rslt}. Aborting. ###${NO_COLOR}" > /dev/stderr
				exit 127
			fi
			echo -e "${DONE_COLOR}Done${NO_COLOR}"
		else
			echo -e "${ERROR_COLOR}File ${FILE_COLOR}${1}${ERROR_COLOR} is not writable. Skipping.${NO_COLOR}"
                        EXIT_VAL=4
		fi
	
	elif [ -f "$1" ]; then
		if [ -w "$1" ]; then
			echo -ne "Shredding ${FILE_COLOR}${1}${NO_COLOR}... "
			echo -ne $ERROR_COLOR
			shred --zero --remove "$1"
			local rslt="$?"
			echo -ne "$NO_COLOR"
			if [ "$rslt" -ne "0" ]; then
				echo -e "${ERROR_COLOR}### ERROR shredding ${FILE_COLOR}${1}${ERROR_COLOR}: shred exited with status ${rslt}. Aborting. ###${NO_COLOR}" > /dev/stderr
				exit 127
			fi
			echo -e "${DONE_COLOR}Done${NO_COLOR}"
		else
			echo -e "${ERROR_COLOR}File ${FILE_COLOR}${1}${ERROR_COLOR} is not writable. Skipping.${NO_COLOR}"
			EXIT_VAL=4
		fi

	else
		echo -e "${ERROR_COLOR}### NOTICE: The file ${FILE_COLOR}${1}${ERROR_COLOR} is not a regular file. Skipping. ###${NO_COLOR}" > /dev/stderr
		EXIT_VAL=3
	fi
}

if [ "$1" = "--no-color" ]; then
	FILE_COLOR=""
	ERROR_COLOR=""
	DONE_COLOR=""
	NO_COLOR=""
	shift
fi

if [ "$#" = "0" -o "$1" = "--help" ]; then
	echo -e "${DONE_COLOR}${verStr}${NO_COLOR}" > /dev/stderr
	echo > /dev/stderr
	echo "Usage: $progName [--no-color] file-or-directory-1 file-or-directory-2 ..." > /dev/stderr
	echo "       $progName --help" > /dev/stderr
	echo "       $progName --version" > /dev/stderr
	EXIT_VAL=1

elif [ "$1" = "--version" ]; then
	echo -e "${DONE_COLOR}${verStr}${NO_COLOR}"
	echo
	licTxt
	EXIT_VAL=2

else
	for i in "$@"; do
		if [ "$i" = "." ]; then i="$(pwd)"; fi
		shredThis "$i"
	done
fi

exit $EXIT_VAL
