#!/usr/bin/env python
# vim: ts=4:sw=4:tw=78:nowrap

#	Copyright (C) 2008 John Zoidberg (KIAaze) <zohn.joidberg@gmail.com>
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.

#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.

#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# A pyGTK app to submit selected hg revisions to svn.
# Usage: hgpush2svn.py [minrev] [maxrev]
# minrev and maxrev are both optional
# It will list revisons from minrev to maxrev, with 0 and last revision being the default values for minrev and maxrev respectively.
# It must be called inside a directory under mercurial and svn control.
# The idea is to be able to use mercurial version control while working offline.
# Note: If you are interested in this, you may also be interested in the "hgsvn" package (doesn't offer revision selection possibility).

import pygtk
pygtk.require("2.0")
import gtk, gobject

import os
import sys
import re

#rev_summary={18:"summary"}
#rev_description={18:"Long description\nOn multiple lines\nJust like this."}
#rev_selected={18:True}

rev_summary={}
rev_description={}
rev_selected={}
rev_files={}

def usage():
	print "A script to commit selected mercurial revisions."
	print 'usage: '+sys.argv[0]+' [minrev] [maxrev]'
	print "By default minrev=0 and maxrev=tip."

def data_extractor():
    print "=== STARTING EXTRACTION ==="
    in_file=open('/tmp/ChangeLog','r')
    summary_reading=False
    description_reading=False
    for line in in_file:
	    if re.findall(r'^changeset:',line):
		summary_reading=False
		description_reading=False
		revnb=int(re.split(':',line)[1])
		print revnb
	    if re.findall(r'^files:',line):
		rev_files[revnb]=line.lstrip('files: ').rstrip('\n')
	    if description_reading:
		print "before:",rev_description[revnb]
		print "Adding:",line
		rev_description[revnb]+=line
		print "after:",rev_description[revnb]
		#sys.exit()
	    if summary_reading:
		rev_summary[revnb]=line
		rev_description[revnb]=line
		print "Summary:",rev_description[revnb]
		summary_reading=False
		description_reading=True
	    if re.findall(r'^description:',line):
		summary_reading=True
    in_file.close()
    for revnb in rev_summary.keys():
	rev_summary[revnb]=rev_summary[revnb].rstrip('\n')
	rev_description[revnb]=rev_description[revnb].rstrip('\n')
	rev_selected[revnb]=True

    print "=== EXTRACTION COMPLETE ==="

def data_verification():
    print "=== STARTING VERIFICATION ==="
    for revnb in rev_summary.keys():
	print "revnb:",revnb
	print "summary:",rev_summary[revnb]
	print "description:"
	print rev_description[revnb]
	print "selected:",rev_selected[revnb]
	print "files:",rev_files[revnb]
	print
    print "=== VERIFICATION COMPLETE ==="

class GUI_Controller:
    """ The GUI class is the controller for our application """
    def __init__(self):
        # setup the main window
        self.root = gtk.Window(type=gtk.WINDOW_TOPLEVEL)
	self.root.set_default_size(500, 500)
        self.root.set_title("hgpush2svn")
        self.root.connect("destroy", self.destroy_cb)
        # Get the model and attach it to the view
        self.mdl = Store.get_model()
        text = self.create_text()

	self.txtview.get_buffer().set_text("HELLO")

	self.view = Display.make_view( self.mdl, self.txtview )
	self.ncols=self.view.get_model().get_n_columns()
	self.nrows=self.view.get_model().iter_n_children(None)

	vbox = gtk.VBox(False, 0)
	self.root.add(vbox)

        # create a vpaned widget and add it to our toplevel window
        vpaned = gtk.VPaned()
	vbox.pack_start(vpaned, True, True, 0)
        #self.root.add(vpaned)
        vpaned.show()

	button = gtk.Button("Commit selected revisions")
	vbox.pack_start(button, False, False, 0)

	button.connect("clicked", self.commit)

        # Now create the contents of the two halves of the window
        #list = self.create_list()

        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        scrolled_window.add_with_viewport (self.view)
        self.view.show()

	vpaned.pack1(scrolled_window,True,True)
        #list.show()

        vpaned.pack2(text,True,True)
        text.show()
        self.root.show()

        # Add our view into the main window
        #self.root.add(self.view)
        self.root.show_all()
        return

    def destroy_cb(self, *kw):
        """ Destroy callback to shutdown the app """
        gtk.main_quit()
        return
    def run(self):
        """ run is called to set off the GTK mainloop """
        gtk.main()
	return	

    # Add some text to our text widget - this is a callback that is invoked
    # when our window is realized. We could also force our window to be
    # realized with GtkWidget.realize, but it would have to be part of a
    # hierarchy first
    def insert_text(self, buffer):
        iter = buffer.get_iter_at_offset(0)
        buffer.insert(iter,
                      "From: pathfinder@nasa.gov\n"
                      "To: mom@nasa.gov\n"
                      "Subject: Made it!\n"
                      "\n"
                      "We just got in this morning. The weather has been\n"
                      "great - clear but cold, and there are lots of fun sights.\n"
                      "Sojourner says hi. See you soon.\n"
                      " -Path\n")
   
    # Create a scrolled text area that displays a "message"
    def create_text(self):
        self.txtview = gtk.TextView()
        buffer = self.txtview.get_buffer()
        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_window.add(self.txtview)
        self.insert_text(buffer)
        scrolled_window.show_all()
        return scrolled_window

    def commit(self, widget):
	print "!!!!!!! WARNING: commit button was clicked !!!!!!!"
	start=self.txtview.get_buffer().get_start_iter()
	end=self.txtview.get_buffer().get_end_iter()
	self.mdl[Display.oldpath][2]=start.get_text(end)
	self.mdl[Display.oldpath][1]=self.mdl[Display.oldpath][2].splitlines()[0]
	#for debugging & checking
	for x in reversed(range(self.nrows)):
	    print "revnb:",self.mdl[x][0]
	    print "summary:",self.mdl[x][1]
	    print "description:"; print self.mdl[x][2]
	    print "selected:",self.mdl[x][3]
	    print "files:",self.mdl[x][4]
	    print
	#Start the real stuff!
	print "=== STARTING COMMIT ==="
	for x in reversed(range(self.nrows)):
	    revnb=self.mdl[x][0]
	    summary=self.mdl[x][1]
	    description=self.mdl[x][2]
	    selected=self.mdl[x][3]
	    files=self.mdl[x][4]
	    if selected:
		cmd="hg revert --all -r"+str(revnb)
		print cmd; os.system(cmd)
		cmd="svn update"
		print cmd; os.system(cmd)
		cmd="svn add "+files
		print cmd; os.system(cmd)
		out_file=open('/tmp/log', 'w')
		out_file.write(description)
		out_file.close()
		print "=== /tmp/log ==="
		os.system('cat /tmp/log')
		print "\n================"
		cmd="svn commit -F /tmp/log"
		print cmd; os.system(cmd)
	print "=== COMMIT COMPLETE ==="

class InfoModel:
    """ The model class holds the information we want to display """
    def __init__(self):
        """ Sets up and populates our gtk.TreeStore """
        self.tree_store = gtk.TreeStore( gobject.TYPE_INT,
					 gobject.TYPE_STRING,
					 gobject.TYPE_STRING,
                                         gobject.TYPE_BOOLEAN,
                                         gobject.TYPE_STRING )
        # places the global people data into the list
        # we form a simple tree.
        for item in reversed(rev_summary.keys()):
            self.tree_store.append( None, (item, rev_summary[item], rev_description[item],rev_selected[item],rev_files[item]) )
            #self.tree_store.append( parent, (tasks[item],None) )
        return
    def get_model(self):
        """ Returns the model """
        if self.tree_store:
            return self.tree_store 
        else:
            return None

class DisplayModel:
    """ Displays the Info_Model model in a view """
    def make_view( self, model, txtview ):
        """ Form a view for the Tree Model """
        self.view = gtk.TreeView( model )
	self.view.connect('cursor-changed',self.col0_selected_cb,model, txtview)
	
        # setup the text cell renderer and allows these
        # cells to be edited.
        self.renderer_revnb = gtk.CellRendererText()
        self.renderer_revnb.set_property( 'editable', False )
        self.renderer_revnb.connect( 'edited', self.col0_edited_cb, model )

        # setup the text cell renderer and allows these
        # cells to be edited.
        self.renderer_summary = gtk.CellRendererText()
        self.renderer_summary.set_property( 'editable', False )
        self.renderer_summary.connect( 'edited', self.col0_edited_cb, model )

        # setup the text cell renderer and allows these
        # cells to be edited.
        self.renderer_description = gtk.CellRendererText()
        self.renderer_description.set_property( 'editable', False )
        self.renderer_description.connect( 'edited', self.col0_edited_cb, model )

        # The toggle cellrenderer is setup and we allow it to be
        # changed (toggled) by the user.
        self.renderer_selected = gtk.CellRendererToggle()
        self.renderer_selected.set_property('activatable', True)
        self.renderer_selected.connect( 'toggled', self.col1_toggled_cb, model )

        # Connect column0 of the display with column 0 in our list model
        # The renderer will then display whatever is in column 0 of
        # our model .
        self.column0 = gtk.TreeViewColumn("Rev #", self.renderer_revnb, text=0)
        self.column1 = gtk.TreeViewColumn("Summary", self.renderer_summary, text=1)

        # The columns active state is attached to the second column
        # in the model.  So when the model says True then the button
        # will show as active e.g on.
        self.column2 = gtk.TreeViewColumn("Commit?", self.renderer_selected )
        self.column2.add_attribute( self.renderer_selected, "active", 3)

	self.column0.set_resizable(True)
	self.column1.set_resizable(True)
	self.column2.set_resizable(True)

	self.view.append_column( self.column0 )
        self.view.append_column( self.column1 )
        self.view.append_column( self.column2 )
	
	self.oldpath=0
	txtview.get_buffer().set_text(model[self.oldpath][2])
	self.view.set_cursor(self.oldpath)

	print "======================="
	ncols=self.view.get_model().get_n_columns()
	nrows=self.view.get_model().iter_n_children(None)
	print "nrows=%d ncols=%d" % (nrows,ncols)
	print "======================="

        return self.view
	
    def col0_selected_cb( self, treeview, model, txtview ):
        """
        Sets the text to the full description of the selected entry
        """
        #model[path][1] = not model[path][1]
        #print "Toggle '%s' to: %s" % (model[path][0], model[path][1],)
	pos=treeview.get_cursor()
	path=pos[0][0]
	print path
        print "Selected '%s'" % (model[path][0])

	#model[self.oldpath][2]
	start=txtview.get_buffer().get_start_iter()
	end=txtview.get_buffer().get_end_iter()
	model[self.oldpath][2]=start.get_text(end)
	model[self.oldpath][1]=model[self.oldpath][2].splitlines()[0]

	print "zozo=",model[self.oldpath][2]
	txtview.get_buffer().set_text(model[path][2])

        print "SELECTED"
	self.oldpath=path
        return

    def col0_edited_cb( self, cell, path, new_text, model ):
        """
        Called when a text cell is edited.  It puts the new text
        in the model so that it is displayed properly.
        """
        print "Change '%s' to '%s'" % (model[path][0], new_text)
        model[path][0] = new_text
        return
    def col1_toggled_cb( self, cell, path, model ):
        """
        Sets the toggled state on the toggle button to true or false.
        """
	print path
        model[path][3] = not model[path][3]
        print "Toggle '%s' to: %s" % (model[path][0], model[path][3],)
        return

if __name__ == '__main__':
    
    #data_extractor()
    #sys.exit()

    os.system("hg tip | awk -F: '/changeset/ {print $2}'>/tmp/stdout")
    f=open('/tmp/stdout','r')
    tip=int(f.readline())
    f.close()

    #for i in reversed(range(0,tip+1)):
	#print i
	#cmd="hg log -r"+str(i)+" | awk -F: '/summary/ {print $2}'"
	#print cmd
	#os.system(cmd)

    print "tip=%d" % tip

    minrev=0
    maxrev=tip
    if len(sys.argv)>1:
	minrev=int(sys.argv[1])
    if len(sys.argv)>2:
	maxrev=int(sys.argv[2])
    print "minrev=%d" % minrev
    print "maxrev=%d" % maxrev

    #rev_summary[18]="toto"
    #rev_description[18]="Hold the line"
    #rev_selected[18]=False
    #rev_summary[42]="the answer..."
    #rev_description[42]="...is always 42."
    #rev_selected[42]=True

    cmd="hg log -v -r%d:%d >/tmp/ChangeLog" % (minrev,maxrev)
    print cmd
    os.system(cmd)

    data_extractor()
    data_verification()

    Store = InfoModel()	
    Display = DisplayModel()
    myGUI = GUI_Controller()
    myGUI.run()
